@@ -10,7 +10,7 @@ option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 the full text of the license.
 '''
 
-import os, os.path, datetime
+import os, os.path, datetime, sys
 
 from packaging_impl import impl as packaging
 
@@ -54,7 +54,8 @@ class CrashDatabase:
         assert dbapi2.paramstyle == 'qmark', \
             'this module assumes qmark dbapi parameter style'
 
-        init = not os.path.exists(path) or path == ':memory:'
+        init = not os.path.exists(path) or path == ':memory:' or \
+            os.path.getsize(path) == 0
         self.duplicate_db = dbapi2.connect(path, timeout=7200)
 
         if init:
@@ -70,6 +71,13 @@ class CrashDatabase:
             cur.execute('''INSERT INTO consolidation VALUES (CURRENT_TIMESTAMP)''')
             self.duplicate_db.commit()
 
+        # verify integrity
+        cur = self.duplicate_db.cursor()
+        cur.execute('PRAGMA integrity_check');
+        result = cur.fetchall() 
+        if result != [('ok',)]:
+            raise SystemError, 'Corrupt duplicate db:' + str(result)
+
     def check_duplicate(self, id, report=None):
         '''Check whether a crash is already known.
 
@@ -210,7 +218,10 @@ class CrashDatabase:
             # crash got fixed/rejected
             fixed_ver = self.get_fixed_version(id)
             if fixed_ver == 'invalid':
+                print 'DEBUG: bug %i was invalidated, removing from database' % id
                 cur2.execute('DELETE FROM crashes WHERE crash_id = ?', [id])
+            elif not fixed_ver:
+                print 'WARNING: inconsistency detected: bug #%i does not appear in get_unfixed(), but is not fixed yet' % id
             else:
                 cur2.execute('UPDATE crashes SET fixed_version = ?, last_change = CURRENT_TIMESTAMP WHERE crash_id = ?',
                     (fixed_ver, id))
@@ -370,9 +381,18 @@ class CrashDatabase:
 
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
-    def close_duplicate(self, id, master):
-        '''Mark a crash id as duplicate of given master ID.'''
+    def duplicate_of(self, id):
+        '''Return master ID for a duplicate bug.
 
+        If the bug is not a duplicate, return None.
+        '''
+        raise NotImplementedError, 'this method must be implemented by a concrete subclass'
+
+    def close_duplicate(self, id, master):
+        '''Mark a crash id as duplicate of given master ID.
+        
+        If master is None, id gets un-duplicated.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def mark_regression(self, id, master):
@@ -386,10 +406,13 @@ class CrashDatabase:
 
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
-    def mark_retrace_failed(self, id):
+    def mark_retrace_failed(self, id, invalid_msg=None):
         '''Mark crash id as 'failed to retrace'.
+
+        If invalid_msg is given, the bug should be closed as invalid with given
+        message, otherwise just marked as a failed retrace.
         
-        This can be a no-op if you are not interested in this, of course.'''
+        This can be a no-op if you are not interested in this.'''
 
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
@@ -436,10 +459,11 @@ def get_crashdb(auth_file, name = None, conf = None):
             if os.path.isfile(cfpath) and cf.endswith('.conf'):
                 try:
                     execfile(cfpath, settings['databases'])
-                except:
+                except Exception, e:
                     # ignore broken files
+                    print >> sys.stderr, 'Invalid file %s: %s' % (cfpath, str(e))
                     pass
-    
+
     if not name:
         name = settings['default']
 
