@@ -50,14 +50,14 @@ class CrashDatabase(apport.crashdb.CrashDatabase):
         return id
 
     def get_comment_url(self, report, handle):
-        '''Return http://<sourcepackage>.bug.net/<handle> for package bugs
-        or http://bug.net/<handle> for reports without a SourcePackage.'''
+        '''Return http://<sourcepackage>.bugs.example.com/<handle> for package bugs
+        or http://bugs.example.com/<handle> for reports without a SourcePackage.'''
 
         if report.has_key('SourcePackage'):
-            return 'http://%s.bug.net/%i' % (report['SourcePackage'],
+            return 'http://%s.bugs.example.com/%i' % (report['SourcePackage'],
                 handle)
         else:
-            return 'http://bug.net/%i' % handle
+            return 'http://bugs.example.com/%i' % handle
 
     def download(self, id):
         '''Download the problem report from given ID and return a Report.'''
@@ -114,9 +114,18 @@ class CrashDatabase(apport.crashdb.CrashDatabase):
         except IndexError:
             return 'invalid'
 
-    def close_duplicate(self, id, master):
-        '''Mark a crash id as duplicate of given master ID.'''
+    def duplicate_of(self, id):
+        '''Return master ID for a duplicate bug.
 
+        If the bug is not a duplicate, return None.
+        '''
+        return self.reports[id]['dup_of']
+
+    def close_duplicate(self, id, master):
+        '''Mark a crash id as duplicate of given master ID.
+        
+        If master is None, id gets un-duplicated.
+        '''
         self.reports[id]['dup_of'] = master
 
     def mark_regression(self, id, master):
@@ -255,7 +264,7 @@ class _MemoryCrashDBTest(unittest.TestCase):
         self.crashes = CrashDatabase(None, None, {'dummy_data': '1'})
 
         self.assertEqual(self.crashes.get_comment_url(self.crashes.download(0),
-            0), 'http://foo.bug.net/0')
+            0), 'http://foo.bugs.example.com/0')
 
         # test-suite internal consistency check: Python signatures are
         # indeed equal and exist
@@ -268,15 +277,14 @@ class _MemoryCrashDBTest(unittest.TestCase):
         self.assertEqual(self.crashes.latest_id(), 4)
 
     def test_no_dummy_data(self):
-        '''Test that no dummy data is added by default.'''
+        '''No dummy data is added by default'''
 
         self.crashes = CrashDatabase(None, None, {})
         self.assertEqual(self.crashes.latest_id(), -1)
         self.assertRaises(IndexError, self.crashes.download, 0)
 
     def test_retrace_markers(self):
-        '''Test bookkeeping in get_unretraced()/mark_retraced() and
-        get_dup_unchecked() and _mark_dup_checked().'''
+        '''Bookkeeping in retraced and dupchecked bugs'''
 
         self.assertEqual(self.crashes.get_unretraced(), set([0, 1, 2]))
         self.assertEqual(self.crashes.get_dup_unchecked(), set([3, 4]))
@@ -286,7 +294,7 @@ class _MemoryCrashDBTest(unittest.TestCase):
     #
 
     def test_submit(self):
-        '''Test crash uploading and downloading.'''
+        '''Crash uploading and downloading'''
 
         # setUp() already checks upload() and get_comment_url()
         r = self.crashes.download(0)
@@ -297,7 +305,7 @@ class _MemoryCrashDBTest(unittest.TestCase):
         self.assertRaises(IndexError, self.crashes.download, 5)
 
     def test_update(self):
-        '''Test update().'''
+        '''update()'''
 
         r = apport.Report()
         r['Package'] = 'new'
@@ -309,25 +317,31 @@ class _MemoryCrashDBTest(unittest.TestCase):
         self.assertRaises(IndexError, self.crashes.update, 5, None)
 
     def test_get_distro_release(self):
-        '''Test get_distro_release().'''
+        '''get_distro_release()'''
 
         self.assertEqual(self.crashes.get_distro_release(0), 'FooLinux Pi/2')
 
     def test_status(self):
-        '''Test get_unfixed(), get_fixed_version() and close_duplicate().'''
+        '''get_unfixed(), get_fixed_version(), duplicate_of(), close_duplicate()'''
 
         self.assertEqual(self.crashes.get_unfixed(), set([0, 1, 2, 4]))
         self.assertEqual(self.crashes.get_fixed_version(0), None)
         self.assertEqual(self.crashes.get_fixed_version(1), None)
         self.assertEqual(self.crashes.get_fixed_version(3), '4.1')
+
+        self.assertEqual(self.crashes.duplicate_of(0), None)
+        self.assertEqual(self.crashes.duplicate_of(1), None)
         self.crashes.close_duplicate(1, 0)
+        self.assertEqual(self.crashes.duplicate_of(0), None)
+        self.assertEqual(self.crashes.duplicate_of(1), 0)
+
         self.assertEqual(self.crashes.get_unfixed(), set([0, 2, 4]))
         self.assertEqual(self.crashes.get_fixed_version(1), 'invalid')
 
         self.assertEqual(self.crashes.get_fixed_version(99), 'invalid')
 
     def test_mark_regression(self):
-        '''Test mark_regression().'''
+        '''mark_regression()'''
 
         self.crashes.mark_regression(4, 3)
         self.assertEqual(self.crashes.reports[4]['comment'], 
@@ -338,7 +352,7 @@ class _MemoryCrashDBTest(unittest.TestCase):
     #
 
     def test_duplicate_db_fixed(self):
-        '''Test duplicate_db_fixed().'''
+        '''duplicate_db_fixed()'''
 
         self.crashes.init_duplicate_db(':memory:')
         self.assertEqual(self.crashes.check_duplicate(0), None)
@@ -352,7 +366,7 @@ class _MemoryCrashDBTest(unittest.TestCase):
             {self.crashes.download(0).crash_signature(): (0, '42')})
 
     def test_duplicate_db_remove(self):
-        '''Test duplicate_db_remove().'''
+        '''duplicate_db_remove()'''
 
         self.crashes.init_duplicate_db(':memory:')
         self.assertEqual(self.crashes.check_duplicate(0), None)
@@ -362,7 +376,7 @@ class _MemoryCrashDBTest(unittest.TestCase):
         self.assertEqual(self.crashes._duplicate_db_dump(), {})
 
     def test_check_duplicate(self):
-        '''Test check_duplicate().'''
+        '''check_duplicate()'''
 
         # db not yet initialized
         self.assertRaises(AssertionError, self.crashes.check_duplicate, 0,
@@ -411,13 +425,13 @@ class _MemoryCrashDBTest(unittest.TestCase):
         # to the correct ID
         r = copy.copy(self.crashes.download(3))
         self.assertEqual(self.crashes.get_comment_url(r, self.crashes.upload(r)),
-            'http://pygoo.bug.net/5')
+            'http://pygoo.bugs.example.com/5')
         self.assertEqual(self.crashes.check_duplicate(5), (3, '4.1'))
 
         r = copy.copy(self.crashes.download(3))
         r['Package'] = 'python-goo 5.1'
         self.assertEqual(self.crashes.get_comment_url(r, self.crashes.upload(r)),
-            'http://pygoo.bug.net/6')
+            'http://pygoo.bugs.example.com/6')
         self.assertEqual(self.crashes.check_duplicate(6), (4, None))
 
         # check with unknown fixed version
@@ -427,14 +441,14 @@ class _MemoryCrashDBTest(unittest.TestCase):
         r = copy.copy(self.crashes.download(3))
         r['Package'] = 'python-goo 5.1'
         self.assertEqual(self.crashes.get_comment_url(r, self.crashes.upload(r)),
-            'http://pygoo.bug.net/7')
+            'http://pygoo.bugs.example.com/7')
         self.assertEqual(self.crashes.check_duplicate(7), (3, ''))
 
         # final consistency check
         self.assertEqual(self.crashes.get_unfixed(), set([0, 2, 4]))
 
     def test_check_duplicate_report_arg(self):
-        '''Test check_duplicate() with explicitly passing report.'''
+        '''check_duplicate() with explicitly passing report'''
 
         self.crashes.init_duplicate_db(':memory:')
 
@@ -450,10 +464,10 @@ class _MemoryCrashDBTest(unittest.TestCase):
 
     # FIXME: fix locking and enable this test
     def __test_duplicate_db_consolidate_race(self):
-        '''Test two parallel instances of duplicate_db_consolidate().
+        '''Two parallel instances of duplicate_db_consolidate()
         
-        One should immediately throw a 'locked' exception.'''
-
+        One should immediately throw a 'locked' exception.
+        '''
         # create db with 1000 unfixed crashes
         self.crashes = CrashDatabase(None, None, {})
         self.crashes.init_duplicate_db(':memory:')
@@ -472,7 +486,7 @@ File "test.py", line 2, in g_foo00
 return x/0
 ZeroDivisionError%i: integer division or modulo by zero''' % bug
             self.assertEqual(self.crashes.get_comment_url(r, self.crashes.upload(r)),
-                'http://pygoo.bug.net/%i' % bug)
+                'http://pygoo.bugs.example.com/%i' % bug)
             self.crashes.check_duplicate(bug)
             # mark crash as fixed now
             self.crashes.reports[bug]['fixed_version'] = str(bug)
@@ -487,7 +501,7 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
             try:
                 self.crashes.duplicate_db_consolidate()
             except Exception, e:
-                if 'database is locked' in e.message:
+                if 'database is locked' in str(e):
                     os._exit(42)
                 else:
                     raise
@@ -496,7 +510,7 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
         try:
             self.crashes.duplicate_db_consolidate()
         except Exception, e:
-            if 'database is locked' in e.message:
+            if 'database is locked' in str(e):
                 locked_exceptions += 1
             else:
                 raise
@@ -517,7 +531,7 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
             self.assertEqual(str(bug), version)
 
     def test_duplicate_db_consolidate(self):
-        '''Test duplicate_db_consolidate().'''
+        '''duplicate_db_consolidate()'''
 
         self.crashes.init_duplicate_db(':memory:')
         self.assertEqual(self.crashes.check_duplicate(0,
@@ -544,7 +558,7 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
              self.crashes.download(3).crash_signature(): (3, '4.1')})
 
     def test_duplicate_db_needs_consolidation(self):
-        '''Test duplicate_db_needs_consolidation().'''
+        '''duplicate_db_needs_consolidation()'''
 
         self.crashes.init_duplicate_db(':memory:')
 
@@ -564,7 +578,7 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
         self.failIf(self.crashes.duplicate_db_needs_consolidation(1))
 
     def test_change_master_id(self):
-        '''Test duplicate_db_change_master_id().'''
+        '''duplicate_db_change_master_id()'''
 
         # db not yet initialized
         self.assertRaises(AssertionError, self.crashes.check_duplicate, 0)
@@ -595,5 +609,34 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
             {self.crashes.download(0).crash_signature(): (0, None),
              self.crashes.download(2).crash_signature(): (99, None)})
 
+    def test_db_corruption(self):
+        '''Detection of DB file corruption'''
+
+        try:
+            (fd, db) = tempfile.mkstemp()
+            os.close(fd)
+            self.crashes.init_duplicate_db(db)
+            self.assertEqual(self.crashes.check_duplicate(0), None)
+            self.assertEqual(self.crashes._duplicate_db_dump(), 
+                {self.crashes.download(0).crash_signature(): (0, None)})
+            self.crashes.duplicate_db_fixed(0, '42')
+            self.assertEqual(self.crashes._duplicate_db_dump(), 
+                {self.crashes.download(0).crash_signature(): (0, '42')})
+
+            self.failIf(self.crashes.duplicate_db_needs_consolidation())
+            del self.crashes
+
+            # damage file
+            f = open(db, 'r+')
+            f.truncate(os.path.getsize(db)*2/3)
+            f.close()
+
+            self.crashes = CrashDatabase(None, None, {})
+            self.assertRaises(SystemError, self.crashes.init_duplicate_db, db)
+
+        finally:
+            os.unlink(db)
+
 if __name__ == '__main__':
+    import tempfile
     unittest.main()
