@@ -10,13 +10,15 @@ option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 the full text of the license.
 '''
 
-import os, glob, subprocess, os.path
+import os, glob, subprocess, os.path, ConfigParser
 from problem_report import ProblemReport
 
 from packaging_impl import impl as packaging
 
 report_dir = os.environ.get('APPORT_REPORT_DIR', '/var/crash')
 
+_config_file = '~/.config/apport/settings'
+
 def find_package_desktopfile(package):
     '''If given package is installed and has a single .desktop file, return the
     path to it, otherwise return None.'''
@@ -216,6 +218,26 @@ def check_files_md5(sumfile):
 
     return mismatches
 
+def get_config(section, setting, default=None, bool=False):
+    '''Return a setting from user configuration.
+
+    This is read from ~/.config/apport/settings. If bool is True, the value is
+    interpreted as a boolean.
+    '''
+    if not get_config.config:
+        get_config.config = ConfigParser.ConfigParser()
+        get_config.config.read(os.path.expanduser(_config_file))
+
+    try:
+        if bool:
+            return get_config.config.getboolean(section, setting)
+        else:
+            return get_config.config.get(section, setting)
+    except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+        return default
+
+get_config.config = None
+
 #
 # Unit test
 #
@@ -226,14 +248,18 @@ from cStringIO import StringIO
 class _ApportUtilsTest(unittest.TestCase):
     def setUp(self):
         global report_dir
+        global _config_file
         self.orig_report_dir = report_dir
         report_dir = tempfile.mkdtemp()
+        self.orig_config_file = _config_file
 
     def tearDown(self):
         global report_dir
+        global _config_file
         shutil.rmtree(report_dir)
         report_dir = self.orig_report_dir
         self.orig_report_dir = None
+        _config_file = self.orig_config_file
 
     def _create_reports(self, create_inaccessible = False):
         '''Create some test reports.'''
@@ -254,7 +280,7 @@ class _ApportUtilsTest(unittest.TestCase):
             return [r1, r2]
 
     def test_find_package_desktopfile(self):
-        '''Test find_package_desktopfile().'''
+        '''find_package_desktopfile().'''
 
         # package without any .desktop file
         nodesktop = 'bash'
@@ -290,7 +316,7 @@ class _ApportUtilsTest(unittest.TestCase):
             self.assert_(d.endswith('.desktop'))
 
     def test_likely_packaged(self):
-        '''Test likely_packaged().'''
+        '''likely_packaged().'''
 
         self.assertEqual(likely_packaged('/bin/bash'), True)
         self.assertEqual(likely_packaged('/usr/bin/foo'), True)
@@ -304,14 +330,14 @@ class _ApportUtilsTest(unittest.TestCase):
         self.assertEqual(likely_packaged('/var/lib/schroot/bin/bash'), False)
 
     def test_find_file_package(self):
-        '''Test find_file_package().'''
+        '''find_file_package().'''
 
         self.assertEqual(find_file_package('/bin/bash'), 'bash')
         self.assertEqual(find_file_package('/bin/cat'), 'coreutils')
         self.assertEqual(find_file_package('/nonexisting'), None)
 
     def test_seen(self):
-        '''Test get_new_reports() and seen_report().'''
+        '''get_new_reports() and seen_report().'''
 
         self.assertEqual(get_new_reports(), [])
         if os.getuid() == 0:
@@ -330,7 +356,7 @@ class _ApportUtilsTest(unittest.TestCase):
             self.assertEqual(set(get_new_reports()), nr)
 
     def test_get_all_reports(self):
-        '''Test get_all_reports().'''
+        '''get_all_reports().'''
 
         self.assertEqual(get_all_reports(), [])
         if os.getuid() == 0:
@@ -346,7 +372,7 @@ class _ApportUtilsTest(unittest.TestCase):
         self.assertEqual(set(get_all_reports()), set(tr))
 
     def test_get_system_reports(self):
-        '''Test get_all_system_reports() and get_new_system_reports().'''
+        '''get_all_system_reports() and get_new_system_reports().'''
 
         self.assertEqual(get_all_reports(), [])
         self.assertEqual(get_all_system_reports(), [])
@@ -367,7 +393,7 @@ class _ApportUtilsTest(unittest.TestCase):
             self.assertEqual(set(get_new_system_reports()), set([]))
 
     def test_delete_report(self):
-        '''Test delete_report().'''
+        '''delete_report().'''
 
         tr = self._create_reports()
 
@@ -376,7 +402,7 @@ class _ApportUtilsTest(unittest.TestCase):
             delete_report(tr.pop())
 
     def test_get_recent_crashes(self):
-        '''Test get_recent_crashes().'''
+        '''get_recent_crashes().'''
 
         # incomplete fields
         r = StringIO('''ProblemType: Crash''')
@@ -405,7 +431,7 @@ CrashCounter: 3''' % time.ctime(time.mktime(time.localtime())-3600))
         self.assertEqual(get_recent_crashes(r), 3)
 
     def test_make_report_path(self):
-        '''Test make_report_path().'''
+        '''make_report_path().'''
 
         pr = ProblemReport()
         self.assertRaises(ValueError, make_report_path, pr)
@@ -416,7 +442,7 @@ CrashCounter: 3''' % time.ctime(time.mktime(time.localtime())-3600))
         self.assert_(make_report_path(pr).startswith('%s/_bin_bash' % report_dir))
 
     def test_check_files_md5(self):
-        '''Test check_files_md5().'''
+        '''check_files_md5().'''
 
         f1 = os.path.join(report_dir, 'test 1.txt')
         f2 = os.path.join(report_dir, 'test:2.txt')
@@ -436,5 +462,43 @@ f6423dfbc4faf022e58b4d3f5ff71a70  %s
         open(f1, 'w').write('Some stuff')
         self.assertEqual(check_files_md5(sumfile), [f2], 'file 2 wrong')
 
+    def test_get_config(self):
+        '''get_config().'''
+
+        global _config_file
+
+        # nonexisting
+        _config_file = '/nonexisting'
+        self.assertEqual(get_config('main', 'foo'), None)
+        self.assertEqual(get_config('main', 'foo', 'moo'), 'moo')
+        get_config.config = None # trash cache
+
+        # empty
+        f = tempfile.NamedTemporaryFile()
+        _config_file = f.name
+        self.assertEqual(get_config('main', 'foo'), None)
+        self.assertEqual(get_config('main', 'foo', 'moo'), 'moo')
+        get_config.config = None # trash cache
+
+        # nonempty
+        f.write('[main]\none=1\ntwo = TWO\nb1 = 1\nb2=False\n[spethial]\none= 99\n')
+        f.flush()
+        self.assertEqual(get_config('main', 'foo'), None)
+        self.assertEqual(get_config('main', 'foo', 'moo'), 'moo')
+        self.assertEqual(get_config('main', 'one'), '1')
+        self.assertEqual(get_config('main', 'one', default='moo'), '1')
+        self.assertEqual(get_config('main', 'two'), 'TWO')
+        self.assertEqual(get_config('main', 'b1', bool=True), True)
+        self.assertEqual(get_config('main', 'b2', bool=True), False)
+        self.assertEqual(get_config('main', 'b3', bool=True), None)
+        self.assertEqual(get_config('main', 'b3', default=False, bool=True), False)
+        self.assertEqual(get_config('spethial', 'one'), '99')
+        self.assertEqual(get_config('spethial', 'two'), None)
+        self.assertEqual(get_config('spethial', 'one', 'moo'), '99')
+        self.assertEqual(get_config('spethial', 'nope', 'moo'), 'moo')
+        get_config.config = None # trash cache
+
+        f.close()
+
 if __name__ == '__main__':
     unittest.main()
