@@ -13,7 +13,7 @@ option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 the full text of the license.
 '''
 
-import glob, sys, os.path, optparse, time, traceback, locale, gettext
+import glob, sys, os.path, optparse, time, traceback, locale, gettext, re
 import pwd, errno, urllib, zlib
 import subprocess, threading, webbrowser
 from gettext import gettext as _
@@ -35,8 +35,8 @@ def thread_collect_info(report, reportfile, package):
         else:
             raise KeyError, 'called without a package, and report does not have ExecutablePath'
     report.add_package_info(package)
-    report.add_hooks_info()
     report.add_os_info()
+    report.add_hooks_info()
 
     # add title
     title = report.standard_title()
@@ -46,9 +46,13 @@ def thread_collect_info(report, reportfile, package):
     # check package origin
     if 'Package' not in report or \
         not apport.packaging.is_distro_package(report['Package'].split()[0]):
-        #TRANS: %s is the name of the operating system
-        report['UnreportableReason'] = _('This is not a genuine %s package') % \
-            report['DistroRelease'].split()[0]
+        if 'APPORT_REPORT_THIRDPARTY' in os.environ or \
+            apport.fileutils.get_config('main', 'thirdparty', False, bool=True):
+            report['ThirdParty'] = 'True'
+        else:
+            #TRANS: %s is the name of the operating system
+            report['UnreportableReason'] = _('This is not a genuine %s package') % \
+                report['DistroRelease'].split()[0]
 
     # check obsolete packages
     if report['ProblemType'] == 'Crash' and \
@@ -142,20 +146,13 @@ class UserInterface:
                     _('unknown program')))
                 heading = _('Sorry, the program "%s" closed unexpectedly') % subject
                 self.ui_error_message(_('Problem in %s') % subject,
-                    "%s\n\n%s" % (heading, _('Your computer does not have enough \
+                    '%s\n\n%s' % (heading, _('Your computer does not have enough \
 free memory to automatically analyze the problem and send a report to the developers.')))
                 return
 
             # check unsupportable flag
             if self.report.has_key('UnsupportableReason'):
-                if self.report.get('ProblemType') == 'Kernel':
-                    subject = _('kernel')
-                elif self.report.get('ProblemType') == 'Package':
-                    subject = self.report['Package']
-                else:
-                    subject = os.path.basename(self.report.get(
-                        'ExecutablePath', _('unknown program')))
-                self.ui_info_message(_('Problem in %s') % subject,
+                self.ui_info_message(_('Unreportable problem'),
                     _('The current configuration cannot be supported:\n\n%s') %
                     self.report['UnsupportableReason'])
                 return
@@ -168,7 +165,13 @@ free memory to automatically analyze the problem and send a report to the develo
                 if response == 'cancel':
                     return
                 assert response == 'report'
-            elif self.report.get('ProblemType') == 'Kernel':
+            elif self.report.get('ProblemType') == 'KernelCrash':
+                response = self.ui_present_kernel_error()
+                if response == 'cancel':
+                    return
+                assert response == 'report'
+            elif self.report.get('ProblemType') == 'KernelOops':
+                # XXX the string doesn't quite match this case
                 response = self.ui_present_kernel_error()
                 if response == 'cancel':
                     return
@@ -221,7 +224,8 @@ free memory to automatically analyze the problem and send a report to the develo
             if self.handle_duplicate():
                 return
 
-            if self.report.get('ProblemType') in ['Crash', 'Kernel']:
+            if self.report.get('ProblemType') in ['Crash', 'KernelCrash',
+                                                  'KernelOops']:
                 response = self.ui_present_report_details()
                 if response == 'cancel':
                     return
@@ -282,12 +286,18 @@ free memory to automatically analyze the problem and send a report to the develo
                 return False
             else:
                 raise
-        self.cur_package = self.options.package
+        if self.options.package:
+            self.options.package = self.options.package.strip()
+        # "Do what I mean" for filing against "linux"
+        if self.options.package == 'linux':
+            self.cur_package = apport.packaging.get_kernel_package()
+        else:
+            self.cur_package = self.options.package
 
         try:
             self.collect_info()
         except ValueError, e:
-            if e.message == 'package does not exist':
+            if str(e) == 'package does not exist':
                 self.ui_error_message(_('Invalid problem report'), 
                     _('Package %s does not exist') % self.cur_package)
                 return False
@@ -301,7 +311,11 @@ free memory to automatically analyze the problem and send a report to the develo
                 del self.report['ProcCmdline']
             except KeyError:
                 pass
-            self.file_report()
+
+            # show what's being sent
+            response = self.ui_present_report_details()
+            if response != 'cancel':
+                self.file_report()
 
         return True
 
@@ -358,7 +372,15 @@ free memory to automatically analyze the problem and send a report to the develo
     def get_complete_size(self):
         '''Return the size of the complete report.'''
 
-        return self.complete_size
+        try:
+            return self.complete_size
+        except AttributeError:
+            # report wasn't loaded, so count manually
+            size = 0
+            for k in self.report:
+                if self.report[k]:
+                    size += len(self.report[k])
+            return size
 
     def get_reduced_size(self):
         '''Return the size of the reduced report.'''
@@ -401,7 +423,7 @@ free memory to automatically analyze the problem and send a report to the develo
             # display a progress dialog
             self.ui_start_info_collection_progress()
 
-            if self.report['ProblemType'] != 'Kernel' and not self.report.has_key('Stacktrace'):
+            if not self.report.has_key('Stacktrace'):
                 icthread = REThread.REThread(target=thread_collect_info,
                     name='thread_collect_info',
                     args=(self.report, self.report_file, self.cur_package))
@@ -414,7 +436,10 @@ free memory to automatically analyze the problem and send a report to the develo
                         sys.exit(1)
                 icthread.exc_raise()
 
-            if self.report['ProblemType'] == 'Kernel' or self.report.has_key('Package'):
+            if self.report.has_key('CrashDB'):
+                self.crashdb = get_crashdb(None, self.report['CrashDB']) 
+
+            if self.report['ProblemType'] == 'KernelCrash' or self.report['ProblemType'] == 'KernelOops' or self.report.has_key('Package'):
                 bpthread = REThread.REThread(target=self.report.search_bug_patterns,
                     args=(self.crashdb.get_bugpattern_baseurl(),))
                 bpthread.start()
@@ -431,8 +456,8 @@ free memory to automatically analyze the problem and send a report to the develo
             self.ui_stop_info_collection_progress()
 
             # check that we were able to determine package names
-            if not self.report.has_key('SourcePackage') or \
-                (self.report['ProblemType'] != 'Kernel' and not self.report.has_key('Package')):
+            if 'SourcePackage' not in self.report or \
+                (not self.report['ProblemType'].startswith('Kernel') and 'Package' not in self.report):
                 self.ui_error_message(_('Invalid problem report'),
                     _('Could not determine the package or source package name.'))
                 # TODO This is not called consistently, is it really needed?
@@ -488,20 +513,27 @@ free memory to automatically analyze the problem and send a report to the develo
                 pass
 
             # if gnome-session is running, try gnome-open; special-case firefox
-            # to open a new window
+            # (and more generally, mozilla browsers) and epiphany to open a new window
+            # with respectively -new-window and --new-window
             try:
                 if os.getenv('DISPLAY') and \
-                        subprocess.call(['pgrep', '-x', '-u', str(uid), 'gnome-session'],
+                        subprocess.call(['pgrep', '-x', '-u', str(uid), 'gnome-panel|gconfd-2'],
                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0:
                     gct = subprocess.Popen(sudo_prefix + ['gconftool', '--get',
                         '/desktop/gnome/url-handlers/http/command'],
                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                    if 'firefox' in gct.communicate()[0] and gct.returncode == 0:
-                        subprocess.call(sudo_prefix + ['firefox', '-new-window', url])
-                        sys.exit(0)
-                    else:
-                        if subprocess.call(sudo_prefix + ['gnome-open', url]) == 0:
+                    if gct.returncode == 0:
+                        preferred_browser = gct.communicate()[0]
+                        browser = re.match('((firefox|seamonkey|flock)[^\s]*)', preferred_browser)
+                        if browser:
+                            subprocess.call(sudo_prefix + [browser.group(0), '-new-window', url])
                             sys.exit(0)
+                        browser = re.match('(epiphany[^\s]*)', preferred_browser)
+                        if browser:
+                            subprocess.call(sudo_prefix + [browser.group(0), '--new-window', url])
+                            sys.exit(0)
+                    if subprocess.call(sudo_prefix + ['gnome-open', url]) == 0:
+                        sys.exit(0)
             except OSError:
                 pass
 
@@ -524,6 +556,13 @@ free memory to automatically analyze the problem and send a report to the develo
         '''Upload the current report to the tracking system and guide the user
         to its web page.'''
 
+        # drop PackageArchitecture if equal to Architecture
+        if self.report.get('PackageArchitecture') == self.report.get('Architecture'):
+            try:
+                del self.report['PackageArchitecture']
+            except KeyError:
+                pass
+
         global __upload_progress
         __upload_progress = None
 
@@ -543,7 +582,7 @@ free memory to automatically analyze the problem and send a report to the develo
                 sys.exit(1)
         if upthread.exc_info():
             self.ui_error_message(_('Network problem'),
-                "%s:\n\n%s" % (
+                '%s:\n\n%s' % (
                     _('Could not upload report data to crash database'),
                     str(upthread.exc_info()[1])
                 ))
@@ -571,7 +610,11 @@ free memory to automatically analyze the problem and send a report to the develo
             self.ui_error_message(_('Memory exhaustion'),
                 _('Your system does not have enough memory to process this crash report.'))
             return False
-        except (TypeError, ValueError, IOError, zlib.error):
+        except IOError, e:
+            self.report = None
+            self.ui_error_message(_('Invalid problem report'), e.strerror)
+            return False
+        except (TypeError, ValueError, zlib.error):
             self.report = None
             self.ui_error_message(_('Invalid problem report'),
                 _('This problem report is damaged and cannot be processed.'))
@@ -583,7 +626,7 @@ free memory to automatically analyze the problem and send a report to the develo
             self.cur_package = apport.fileutils.find_file_package(self.report.get('ExecutablePath', ''))
 
         exe_path = self.report.get('InterpreterPath', self.report.get('ExecutablePath'))
-        if not self.cur_package and self.report['ProblemType'] != 'Kernel' or (
+        if not self.cur_package and self.report['ProblemType'] != 'KernelCrash' and self.report['ProblemType'] != 'KernelOops' or (
             exe_path and not os.path.exists(exe_path)):
             msg = _('This problem report does not apply to a packaged program.')
             if self.report.has_key('ExecutablePath'):
@@ -658,7 +701,7 @@ might be helpful for the developers.'))
         raise NotImplementedError, 'this function must be overridden by subclasses'
 
     def ui_present_kernel_error(self, desktopentry):
-        '''Inform that a kernel Oops has happened for self.report and
+        '''Inform that a kernel crash has happened for self.report and
         ask about an action.
 
         Return the action: ignore ('cancel'), or report a bug about the problem
@@ -885,7 +928,7 @@ databases = {
             self.assertEqual(subprocess.call(['pidof', '/bin/sleep']), 1, 'no stray sleeps')
 
         def test_format_filesize(self):
-            '''Test format_filesize().'''
+            '''format_filesize().'''
 
             self.assertEqual(self.ui.format_filesize(0), '0.0 KiB')
             self.assertEqual(self.ui.format_filesize(2048), '2.0 KiB')
@@ -896,8 +939,8 @@ databases = {
             self.assertEqual(self.ui.format_filesize(1024*1048576), '1.0 GiB')
             self.assertEqual(self.ui.format_filesize(2560*1048576), '2.5 GiB')
 
-        def test_get_size(self):
-            '''Test get_complete_size() and get_reduced_size().'''
+        def test_get_size_loaded(self):
+            '''get_complete_size() and get_reduced_size() for loaded Reports.'''
 
             self.ui.load_report(self.report_file.name)
 
@@ -907,8 +950,20 @@ databases = {
             self.assert_(rs > 1000)
             self.assert_(rs < 10000)
 
+        def test_get_size_constructed(self):
+            '''get_complete_size() and get_reduced_size() for on-the-fly Reports.'''
+
+            self.ui.report = apport.Report('Bug')
+            self.ui.report['Hello'] = 'World'
+
+            s = self.ui.get_complete_size()
+            self.assert_(s > 5)
+            self.assert_(s < 100)
+
+            self.assertEqual(s, self.ui.get_reduced_size())
+
         def test_load_report(self):
-            '''Test load_report().'''
+            '''load_report().'''
 
             # valid report
             self.ui.load_report(self.report_file.name)
@@ -943,7 +998,7 @@ CoreDump: base64
             self.assertEqual(self.ui.msg_severity, 'error')
 
         def test_restart(self):
-            '''Test restart().'''
+            '''restart().'''
 
             # test with only ProcCmdline
             p = os.path.join(apport.fileutils.report_dir, 'ProcCmdline')
@@ -976,7 +1031,7 @@ CoreDump: base64
             self.ui.load_report(self.report_file.name)
 
         def test_collect_info_distro(self):
-            '''Test collect_info() on report without information (distro bug).'''
+            '''collect_info() on report without information (distro bug).'''
 
             # report without any information (distro bug)
             self.ui.report = apport.Report()
@@ -987,7 +1042,7 @@ CoreDump: base64
                 'no progress dialog for distro bug info collection')
 
         def test_collect_info_exepath(self):
-            '''Test collect_info() on report with only ExecutablePath.'''
+            '''collect_info() on report with only ExecutablePath.'''
 
             # report with only package information
             self.report = apport.Report()
@@ -1008,7 +1063,7 @@ CoreDump: base64
                 'progress dialog for package bug info collection finished')
 
         def test_collect_info_package(self):
-            '''Test collect_info() on report with a package.'''
+            '''collect_info() on report with a package.'''
 
             # report with only package information
             self.ui.report = apport.Report()
@@ -1023,7 +1078,7 @@ CoreDump: base64
                 'progress dialog for package bug info collection finished')
 
         def test_handle_duplicate(self):
-            '''Test handle_duplicate().'''
+            '''handle_duplicate().'''
 
             self.ui.load_report(self.report_file.name)
             self.assertEqual(self.ui.handle_duplicate(), False)
@@ -1039,14 +1094,14 @@ CoreDump: base64
             self.assertEqual(self.ui.opened_url, demo_url)
 
         def test_run_nopending(self):
-            '''Test running the frontend without any pending reports.'''
+            '''running the frontend without any pending reports.'''
 
             sys.argv = []
             self.ui = _TestSuiteUserInterface()
             self.assertEqual(self.ui.run_argv(), False)
 
         def test_run_report_bug_noargs(self):
-            '''Test run_report_bug() without specifying arguments.'''
+            '''run_report_bug() without specifying arguments.'''
 
             sys.argv = ['ui-test', '-f']
             self.ui = _TestSuiteUserInterface()
@@ -1054,7 +1109,7 @@ CoreDump: base64
             self.assertEqual(self.ui.msg_severity, 'error')
 
         def test_run_report_bug_package(self):
-            '''Test run_report_bug() for a package.'''
+            '''run_report_bug() for a package.'''
 
             sys.argv = ['ui-test', '-f', '-p', 'bash']
             self.ui = _TestSuiteUserInterface()
@@ -1062,7 +1117,7 @@ CoreDump: base64
 
             self.assertEqual(self.ui.msg_severity, None)
             self.assertEqual(self.ui.msg_title, None)
-            self.assertEqual(self.ui.opened_url, 'http://bash.bug.net/%i' % self.ui.crashdb.latest_id())
+            self.assertEqual(self.ui.opened_url, 'http://bash.bugs.example.com/%i' % self.ui.crashdb.latest_id())
 
             self.assert_(self.ui.ic_progress_pulses > 0)
             self.assertEqual(self.ui.report['SourcePackage'], 'bash')
@@ -1078,7 +1133,7 @@ CoreDump: base64
             self.assertEqual(self.ui.msg_severity, 'error')
 
         def test_run_report_bug_pid(self):
-            '''Test run_report_bug() for a pid.'''
+            '''run_report_bug() for a pid.'''
 
             # fork a test process
             pid = os.fork()
@@ -1108,11 +1163,11 @@ CoreDump: base64
 
             self.assertEqual(self.ui.msg_severity, None)
             self.assertEqual(self.ui.msg_title, None)
-            self.assertEqual(self.ui.opened_url, 'http://coreutils.bug.net/%i' % self.ui.crashdb.latest_id())
+            self.assertEqual(self.ui.opened_url, 'http://coreutils.bugs.example.com/%i' % self.ui.crashdb.latest_id())
             self.assert_(self.ui.ic_progress_pulses > 0)
 
         def test_run_report_bug_wrong_pid(self):
-            '''Test run_report_bug() for a nonexisting pid.'''
+            '''run_report_bug() for a nonexisting pid.'''
 
             # search an unused pid
             pid = 1
@@ -1131,7 +1186,7 @@ CoreDump: base64
             self.ui.run_argv()
 
         def test_run_report_bug_noperm_pid(self):
-            '''Test run_report_bug() for a pid which runs as a different user.'''
+            '''run_report_bug() for a pid which runs as a different user.'''
 
             assert os.getuid() > 0, 'this test must not be run as root'
 
@@ -1142,14 +1197,18 @@ CoreDump: base64
             self.assertEqual(self.ui.msg_severity, 'error')
 
         def test_run_report_bug_unpackaged_pid(self):
-            '''Test run_report_bug() for a pid of an unpackaged program.'''
+            '''run_report_bug() for a pid of an unpackaged program.'''
+
+            # create unpackaged test program
+            (fd, exename) = tempfile.mkstemp()
+            os.write(fd, open('/bin/cat').read())
+            os.close(fd)
+            os.chmod(exename, 0755)
 
             # unpackaged test process
             pid = os.fork()
             if pid == 0:
-                sys.argv = ['/tmp/foo']
-                time.sleep(3600)
-                os._exit(0)
+                os.execv(exename, [exename])
 
             try:
                 sys.argv = ['ui-test', '-f', '-P', str(pid)]
@@ -1158,6 +1217,7 @@ CoreDump: base64
             finally:
                 os.kill(pid, signal.SIGKILL)
                 os.wait()
+                os.unlink(exename)
 
             self.assertEqual(self.ui.msg_severity, 'error')
 
@@ -1196,7 +1256,7 @@ CoreDump: base64
             return r
 
         def test_run_crash(self):
-            '''Test run_crash().'''
+            '''run_crash().'''
 
             r = self._gen_test_crash()
 
@@ -1233,7 +1293,7 @@ CoreDump: base64
             self.ui.run_crash(report_file)
             self.assertEqual(self.ui.msg_severity, None)
             self.assertEqual(self.ui.msg_title, None)
-            self.assertEqual(self.ui.opened_url, 'http://coreutils.bug.net/%i' % self.ui.crashdb.latest_id())
+            self.assertEqual(self.ui.opened_url, 'http://coreutils.bugs.example.com/%i' % self.ui.crashdb.latest_id())
             self.assertNotEqual(self.ui.ic_progress_pulses, 0)
 
             self.assert_('SourcePackage' in self.ui.report.keys())
@@ -1252,7 +1312,7 @@ CoreDump: base64
             self.ui.run_crash(report_file)
             self.assertEqual(self.ui.msg_severity, None)
             self.assertEqual(self.ui.msg_title, None)
-            self.assertEqual(self.ui.opened_url, 'http://coreutils.bug.net/%i' % self.ui.crashdb.latest_id())
+            self.assertEqual(self.ui.opened_url, 'http://coreutils.bugs.example.com/%i' % self.ui.crashdb.latest_id())
             self.assertNotEqual(self.ui.ic_progress_pulses, 0)
 
             self.assert_('SourcePackage' in self.ui.report.keys())
@@ -1277,7 +1337,7 @@ CoreDump: base64
             self.assert_(self.ui.report.check_ignored())
 
         def test_run_crash_argv_file(self):
-            '''Test run_crash() through a file specified on the command line.'''
+            '''run_crash() through a file specified on the command line.'''
 
             self.report['Package'] = 'bash'
             self.report['UnsupportableReason'] = 'It stinks.'
@@ -1298,7 +1358,7 @@ CoreDump: base64
             self.assertEqual(self.ui.msg_severity, 'error')
 
         def test_run_crash_unsupportable(self):
-            '''Test run_crash() on a crash with the UnsupportableReason
+            '''run_crash() on a crash with the UnsupportableReason
             field.'''
 
             self.report['UnsupportableReason'] = 'It stinks.'
@@ -1312,7 +1372,7 @@ CoreDump: base64
             self.assertEqual(self.ui.msg_severity, 'info')
 
         def test_run_crash_unreportable(self):
-            '''Test run_crash() on a crash with the UnreportableReason
+            '''run_crash() on a crash with the UnreportableReason
             field.'''
 
             self.report['UnreportableReason'] = 'It stinks.'
@@ -1329,7 +1389,7 @@ CoreDump: base64
             self.assertEqual(self.ui.msg_severity, 'info')
 
         def test_run_crash_ignore(self):
-            '''Test run_crash() on a crash with the Ignore field.'''
+            '''run_crash() on a crash with the Ignore field.'''
 
             self.report['Ignore'] = 'True'
             self.report['ExecutablePath'] = '/bin/bash'
@@ -1340,7 +1400,7 @@ CoreDump: base64
             self.assertEqual(self.ui.msg_severity, None)
 
         def test_run_crash_nocore(self):
-            '''Test run_crash() for a crash dump without CoreDump.'''
+            '''run_crash() for a crash dump without CoreDump.'''
 
             # create a test executable
             test_executable = '/bin/cat'
@@ -1376,7 +1436,7 @@ CoreDump: base64
                 (self.ui.msg_title, self.ui.msg_text))
 
         def test_run_crash_preretraced(self):
-            '''Test run_crash() pre-retraced reports.
+            '''run_crash() pre-retraced reports.
             
             This happens with crashes which are pre-processed by
             apport-retrace.'''
@@ -1403,7 +1463,7 @@ CoreDump: base64
             self.assertEqual(self.ui.ic_progress_pulses, 0)
            
         def test_run_crash_errors(self):
-            '''Test run_crash() on various error conditions.'''
+            '''run_crash() on various error conditions.'''
 
             # crash report with invalid Package name
             r = apport.Report()
@@ -1419,7 +1479,7 @@ CoreDump: base64
             self.assertEqual(self.ui.msg_severity, 'error')
 
         def test_run_crash_uninstalled(self):
-            '''Test run_crash() on reports with subsequently uninstalled packages'''
+            '''run_crash() on reports with subsequently uninstalled packages'''
 
             # program got uninstalled between crash and report
             r = self._gen_test_crash()
@@ -1457,7 +1517,7 @@ CoreDump: base64
             self.assertEqual(self.ui.msg_severity, 'info')
 
         def test_run_crash_package(self):
-            '''Test run_crash() for a package error.'''
+            '''run_crash() for a package error.'''
 
             # generate crash report
             r = apport.Report('Package')
@@ -1487,24 +1547,24 @@ CoreDump: base64
             self.ui.run_crash(report_file)
             self.assertEqual(self.ui.msg_severity, None)
             self.assertEqual(self.ui.msg_title, None)
-            self.assertEqual(self.ui.opened_url, 'http://bash.bug.net/%i' % self.ui.crashdb.latest_id())
+            self.assertEqual(self.ui.opened_url, 'http://bash.bugs.example.com/%i' % self.ui.crashdb.latest_id())
 
             self.assert_('SourcePackage' in self.ui.report.keys())
             self.assert_('Package' in self.ui.report.keys())
             self.assertEqual(self.ui.report['ProblemType'], 'Package')
 
             # verify that additional information has been collected
-            self.assert_('PackageArchitecture' in self.ui.report.keys())
+            self.assert_('Architecture' in self.ui.report.keys())
             self.assert_('DistroRelease' in self.ui.report.keys())
             self.assert_('Uname' in self.ui.report.keys())
 
         def test_run_crash_kernel(self):
-            '''Test run_crash() for a kernel error.'''
+            '''run_crash() for a kernel error.'''
 
             # generate crash report
-            r = apport.Report('Kernel')
-            r['SourcePackage'] = 'linux-source-2.6.20'
-            r.add_os_info()
+            r = apport.Report('KernelCrash')
+            r['Package'] = apport.packaging.get_kernel_package()
+            r['SourcePackage'] = 'linux'
 
             # write crash report
             report_file = os.path.join(apport.fileutils.report_dir, 'test.crash')
@@ -1526,15 +1586,19 @@ CoreDump: base64
             self.ui.present_kernel_error_response = 'report'
             self.ui.present_details_response = 'full'
             self.ui.run_crash(report_file)
-            self.assertEqual(self.ui.msg_severity, None)
+            self.assertEqual(self.ui.msg_severity, None, str(self.ui.msg_title) + 
+                ' ' + str(self.ui.msg_text))
             self.assertEqual(self.ui.msg_title, None)
-            self.assertEqual(self.ui.opened_url, 'http://linux-source-2.6.20.bug.net/%i' % self.ui.crashdb.latest_id())
+            self.assertEqual(self.ui.opened_url, 'http://linux.bugs.example.com/%i' % self.ui.crashdb.latest_id())
 
             self.assert_('SourcePackage' in self.ui.report.keys())
-            self.assertEqual(self.ui.report['ProblemType'], 'Kernel')
+            # did we run the hooks properly?
+            self.assert_('ProcModules' in self.ui.report.keys())
+            self.assert_('Lspci' in self.ui.report.keys())
+            self.assertEqual(self.ui.report['ProblemType'], 'KernelCrash')
 
         def test_run_crash_anonymity(self):
-            '''Test run_crash() anonymization.'''
+            '''run_crash() anonymization.'''
 
             r = self._gen_test_crash()
             report_file = os.path.join(apport.fileutils.report_dir, 'test.crash')
