@@ -98,17 +98,18 @@ class ProblemReport(UserDict.IterableUserDict):
         self.old_keys = set()
 
     def load(self, file, binary=True):
-        '''Initialize problem report from a file-like object, using Debian
-        control file format.
-
-        if binary is False, binary data is not loaded; the dictionary key is
+        '''Initialize problem report from a file-like object.
+        
+        If binary is False, binary data is not loaded; the dictionary key is
         created, but its value will be an empty string. If it is true, it is
         transparently uncompressed and available as dictionary string values.
         If binary is 'compressed', the compressed value is retained, and the
         dictionary value will be a CompressedValue object. This is useful if
         the compressed value is still useful (to avoid recompression if the
-        file needs to be written back).'''
+        file needs to be written back).
 
+        Files are in RFC822 format.
+        '''
         self.data.clear()
         key = None
         value = None
@@ -185,8 +186,7 @@ class ProblemReport(UserDict.IterableUserDict):
         return False
 
     def write(self, file, only_new = False):
-        '''Write information into the given file-like object, using Debian
-        control file format.
+        '''Write information into the given file-like object.
 
         If only_new is True, only keys which have been added since the last
         load() are written (i. e. those returned by new_keys()).
@@ -199,8 +199,9 @@ class ProblemReport(UserDict.IterableUserDict):
         gzip compressed and base64-encoded (this defaults to True). If limit is
         set to a positive integer, the entire key will be removed. If
         fail_on_empty is True, reading zero bytes will cause an IOError.
-        '''
 
+        Files are written in RFC822 format.
+        '''
         # sort keys into ASCII non-ASCII/binary attachment ones, so that
         # the base64 ones appear last in the report
         asckeys = []
@@ -251,6 +252,10 @@ class ProblemReport(UserDict.IterableUserDict):
                     del self.data[k]
                     continue
 
+            if type(v) == type(u''):
+                # unicode → str
+                v = v.encode('UTF-8')
+
             if '\n' in v:
                 # multiline value
                 print >> file, k + ':'
@@ -437,8 +442,13 @@ class ProblemReport(UserDict.IterableUserDict):
                 attachments.append(att)
             else:
                 # plain text value
+                if type(v) == type(u''):
+                    # convert unicode to UTF-8 str
+                    v = v.encode('UTF-8')
+
                 lines = len(v.splitlines())
                 if lines == 1:
+                    v = v.rstrip()
                     text += '%s: %s\n' % (k, v)
                 elif lines <= attach_treshold:
                     text += '%s:\n ' % k
@@ -511,7 +521,7 @@ import unittest, tempfile, os, email
 
 class _ProblemReportTest(unittest.TestCase):
     def test_basic_operations(self):
-        '''Test basic creation and operation.'''
+        '''basic creation and operation.'''
 
         pr = ProblemReport()
         pr['foo'] = 'bar'
@@ -522,7 +532,7 @@ class _ProblemReportTest(unittest.TestCase):
         self.assert_(time.strptime(pr['Date']))
 
     def test_ctor_arguments(self):
-        '''Test non-default constructor arguments.'''
+        '''non-default constructor arguments.'''
 
         pr = ProblemReport('KernelCrash')
         self.assertEqual(pr['ProblemType'], 'KernelCrash')
@@ -530,7 +540,7 @@ class _ProblemReportTest(unittest.TestCase):
         self.assertEqual(pr['Date'], '19801224 12:34')
 
     def test_sanity_checks(self):
-        '''Test various error conditions.'''
+        '''various error conditions.'''
 
         pr = ProblemReport()
         self.assertRaises(AssertionError, pr.__setitem__, 'a b', '1')
@@ -541,7 +551,7 @@ class _ProblemReportTest(unittest.TestCase):
         self.assertRaises(KeyError, pr.__getitem__, 'Nonexistant')
 
     def test_compressed_values(self):
-        '''Test handling of CompressedValue values.'''
+        '''handling of CompressedValue values.'''
 
         large_val = 'A' * 5000000
 
@@ -582,10 +592,14 @@ class _ProblemReportTest(unittest.TestCase):
         self.assertEqual(pr['Large'], large_val)
 
     def test_write(self):
-        '''Test write() and proper formatting.'''
+        '''write() and proper formatting.'''
 
         pr = ProblemReport(date = 'now!')
         pr['Simple'] = 'bar'
+        pr['SimpleUTF8'] = '1äö2Φ3'
+        pr['SimpleUnicode'] = u'1äö2Φ3'
+        pr['TwoLineUTF8'] = 'pi-π\nnu-η'
+        pr['TwoLineUnicode'] = u'pi-π\nnu-η'
         pr['WhiteSpace'] = ' foo   bar\nbaz\n  blip  \n\nafteremptyline'
         io = StringIO()
         pr.write(io)
@@ -593,6 +607,14 @@ class _ProblemReportTest(unittest.TestCase):
 '''ProblemType: Crash
 Date: now!
 Simple: bar
+SimpleUTF8: 1äö2Φ3
+SimpleUnicode: 1äö2Φ3
+TwoLineUTF8:
+ pi-π
+ nu-η
+TwoLineUnicode:
+ pi-π
+ nu-η
 WhiteSpace:
   foo   bar
  baz
@@ -602,7 +624,7 @@ WhiteSpace:
 ''')
 
     def test_write_append(self):
-        '''Test write() with appending to an existing file.'''
+        '''write() with appending to an existing file.'''
 
         pr = ProblemReport(date = 'now!')
         pr['Simple'] = 'bar'
@@ -648,7 +670,7 @@ Extra: appended
         self.assertEqual(pr['Extra'], 'appended')
 
     def test_load(self):
-        '''Test load() with various formatting.'''
+        '''load() with various formatting.'''
         pr = ProblemReport()
         pr.load(StringIO(
 '''ProblemType: Crash
@@ -719,7 +741,7 @@ Last: foo
         self.assertEqual(pr.keys(), ['ProblemType'])
 
     def test_write_file(self):
-        '''Test writing a report with binary file data.'''
+        '''writing a report with binary file data.'''
 
         temp = tempfile.NamedTemporaryFile()
         temp.write('AB' * 10 + '\0' * 10 + 'Z')
@@ -772,7 +794,7 @@ File: base64
         temp.close()
 
     def test_write_fileobj(self):
-        '''Test writing a report with a pointer to a file-like object.'''
+        '''writing a report with a pointer to a file-like object.'''
 
         tempbin = StringIO('AB' * 10 + '\0' * 10 + 'Z')
         tempasc = StringIO('Hello World')
@@ -790,7 +812,7 @@ File: base64
         self.assertEqual(pr['AscFile'], tempasc.getvalue())
 
     def test_write_empty_fileobj(self):
-        '''Test writing a report with a pointer to a file-like object with enforcing non-emptyness.'''
+        '''writing a report with a pointer to a file-like object with enforcing non-emptyness.'''
 
         tempbin = StringIO('')
         tempasc = StringIO('')
@@ -806,7 +828,7 @@ File: base64
         self.assertRaises(IOError, pr.write, io)
 
     def test_write_delayed_fileobj(self):
-        '''Test writing a report with file pointers and delayed data.'''
+        '''writing a report with file pointers and delayed data.'''
 
         (fout, fin) = os.pipe()
 
@@ -837,7 +859,7 @@ File: base64
         self.assertEqual(len(pr2['BinFile']), 1048576 + len('hello world'))
 
     def test_read_file(self):
-        '''Test reading a report with binary data.'''
+        '''reading a report with binary data.'''
 
         bin_report = '''ProblemType: Crash
 Date: now!
@@ -867,7 +889,7 @@ Foo: Bar
         self.assertEqual(pr['File'].get_value(), 'AB' * 10 + '\0' * 10 + 'Z')
 
     def test_read_file_legacy(self):
-        '''Test reading a report with binary data in legacy format without gzip
+        '''reading a report with binary data in legacy format without gzip
         header.'''
 
         bin_report = '''ProblemType: Crash
@@ -902,7 +924,7 @@ Foo: Bar
         self.assertEqual(io.read(), data)
 
     def test_big_file(self):
-        '''Test writing and re-decoding a big random file.'''
+        '''writing and re-decoding a big random file.'''
 
         # create 1 MB random file
         temp = tempfile.NamedTemporaryFile()
@@ -940,7 +962,7 @@ Foo: Bar
         self.assertEqual(pr['File'].get_value(), data)
 
     def test_size_limit(self):
-        '''Test writing and a big random file with a size limit key.'''
+        '''writing and a big random file with a size limit key.'''
 
         # create 1 MB random file
         temp = tempfile.NamedTemporaryFile()
@@ -961,8 +983,6 @@ Foo: Bar
         pr.write(io)
         temp.close()
 
-        open('/tmp/r', 'w').write(io.getvalue())
-
         # read it again
         io.seek(0)
         pr = ProblemReport()
@@ -977,7 +997,7 @@ Foo: Bar
         self.assertEqual(pr['ZAfter'], 'ytesty')
 
     def test_iter(self):
-        '''Test ProblemReport iteration.'''
+        '''ProblemReport iteration.'''
 
         pr = ProblemReport()
         pr['foo'] = 'bar'
@@ -991,7 +1011,7 @@ Foo: Bar
         self.assertEqual(len([k for k in pr if k != 'foo']), 2)
 
     def test_modify(self):
-        '''Test reading, modifying fields, and writing back.'''
+        '''reading, modifying fields, and writing back.'''
 
         report = '''ProblemType: Crash
 Date: now!
@@ -1032,7 +1052,7 @@ File: base64
 ''')
 
     def test_add_to_existing(self):
-        '''Test adding information to an existing report.'''
+        '''adding information to an existing report.'''
 
         # original report
         pr = ProblemReport()
@@ -1107,11 +1127,16 @@ File: base64
         os.unlink(rep)
 
     def test_write_mime_text(self):
-        '''Test write_mime() for text values.'''
+        '''write_mime() for text values.'''
 
         pr = ProblemReport(date = 'now!')
         pr['Simple'] = 'bar'
+        pr['SimpleUTF8'] = '1äö2Φ3'
+        pr['SimpleUnicode'] = u'1äö2Φ3'
+        pr['SimpleLineEnd'] = 'bar\n'
         pr['TwoLine'] = 'first\nsecond\n'
+        pr['TwoLineUTF8'] = 'pi-π\nnu-η\n'
+        pr['TwoLineUnicode'] = u'pi-π\nnu-η\n'
         pr['InlineMargin'] = 'first\nsecond\nthird\nfourth\nfifth\n'
         pr['Multiline'] = ' foo   bar\nbaz\n  blip  \nline4\nline♥5!!\nłıµ€ ⅝\n'
         io = StringIO()
@@ -1141,9 +1166,18 @@ InlineMargin:
  fourth
  fifth
 Simple: bar
+SimpleLineEnd: bar
+SimpleUTF8: 1äö2Φ3
+SimpleUnicode: 1äö2Φ3
 TwoLine:
  first
  second
+TwoLineUTF8:
+ pi-π
+ nu-η
+TwoLineUnicode:
+ pi-π
+ nu-η
 ''')
 
         # third part should be the Multiline: field as attachment
@@ -1164,7 +1198,7 @@ line♥5!!
         self.assertRaises(StopIteration, msg_iter.next)
 
     def test_write_mime_binary(self):
-        '''Test write_mime() for binary values and file references.'''
+        '''write_mime() for binary values and file references.'''
 
         bin_value = 'AB' * 10 + '\0' * 10 + 'Z'
 
@@ -1263,7 +1297,7 @@ line♥5!!
         self.assertRaises(StopIteration, msg_iter.next)
 
     def test_write_mime_extra_headers(self):
-        '''Test write_mime() with extra headers.'''
+        '''write_mime() with extra headers.'''
 
         pr = ProblemReport(date = 'now!')
         pr['Simple'] = 'bar'
@@ -1293,7 +1327,7 @@ line♥5!!
         self.assertRaises(StopIteration, msg_iter.next)
 
     def test_write_mime_filter(self):
-        '''Test write_mime() with key filters.'''
+        '''write_mime() with key filters.'''
 
         bin_value = 'AB' * 10 + '\0' * 10 + 'Z'
 
@@ -1337,7 +1371,7 @@ GoodText: Hi
         self.assertRaises(StopIteration, msg_iter.next)
 
     def test_updating(self):
-        '''Test new_keys() and write() with only_new=True.'''
+        '''new_keys() and write() with only_new=True.'''
 
         pr = ProblemReport()
         self.assertEqual(pr.new_keys(), set(['ProblemType', 'Date']))
@@ -1359,7 +1393,7 @@ Baz: blob
         self.assertEqual(out.getvalue(), 'NewKey: new new\n')
 
     def test_import_dict(self):
-        '''Test importing a dictionary with update().'''
+        '''importing a dictionary with update().'''
 
         pr = ProblemReport()
         pr['oldtext'] = 'Hello world'
